//********************************
//**ScummVM Quick And Easy 1.4.0*
//**By Ben Gorman (Bgbennyboy)****
//**Http://quick.mixnmojo.com/****
//********************************

unit GameFilters;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, JvExStdCtrls, JvCheckBox, ExtCtrls, jclregistry, registry;

type
  TFiltersfrm = class(TForm)
    GroupBoxFilters: TGroupBox;
    chkboxInstalled: TJvCheckBox;
    chkboxSCUMM: TJvCheckBox;
    chkboxHumongous: TJvCheckBox;
    chkboxRevolution: TJvCheckBox;
    chkboxOther: TJvCheckBox;
    chkboxFull: TJvCheckBox;
    chkboxDemos: TJvCheckBox;
    listboxFilters: TListBox;
    lblFilters: TLabel;
    editFilterName: TLabeledEdit;
    btnDelete: TButton;
    btnNew: TButton;
    procedure btnNewClick(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
    procedure listboxFiltersClick(Sender: TObject);
    procedure editFilterNameChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure chkboxSCUMMClick(Sender: TObject);
    procedure chkboxRevolutionClick(Sender: TObject);
    procedure chkboxFullClick(Sender: TObject);
    procedure chkboxInstalledClick(Sender: TObject);
    procedure chkboxHumongousClick(Sender: TObject);
    procedure chkboxOtherClick(Sender: TObject);
    procedure chkboxDemosClick(Sender: TObject);
  private
    procedure PopulateFilters;
    procedure DeleteWonkyFilterKey(keyname: string);
    procedure LoadFilterSettings(index: integer);
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Filtersfrm: TFiltersfrm;
  keyname: string;
  
implementation

{$R *.dfm}

procedure TFiltersfrm.btnNewClick(Sender: TObject);
var
  newname: integer;
begin
  listboxFilters.Items.Add('New Filter');
  newname:=-1;
  repeat
    inc(newname);
  until regkeyexists(HKEY_CURRENT_USER,'Software\Quick And Easy\ScummVM\filters\' + inttostr(newname))=false;

  regcreatekey(HKEY_CURRENT_USER,'Software\Quick And Easy\ScummVM\filters\' + inttostr(newname));
  regwritestring(HKEY_CURRENT_USER,'Software\Quick And Easy\ScummVM\filters\' + inttostr(newname), 'filtername', 'New Filter');
end;

procedure TFiltersfrm.btnDeleteClick(Sender: TObject);
begin
  if listboxFilters.ItemIndex = -1 then
    exit;

  DeleteWonkyFilterKey(keyname);
  listboxFilters.Items.Delete(listboxFilters.ItemIndex);
  editFilterName.Text:='';
  editFilterName.Enabled:=false;
  GroupBoxFilters.Enabled:=false;
  btnDelete.Enabled:=false;
  chkboxFull.Checked:=false;
  chkboxHumongous.Checked:=false;
  chkboxDemos.Checked:=false;
  chkboxInstalled.checked:=false;
  chkboxOther.Checked:=false;
  chkboxRevolution.Checked:=false;
  chkboxSCUMM.Checked:=false;
end;

procedure TFiltersfrm.listboxFiltersClick(Sender: TObject);
begin
  if listboxFilters.ItemIndex = -1 then
    exit;
    
  keyname:='';
  GroupBoxFilters.Enabled:=true;
  editFilterName.Enabled:=true;
  btnDelete.Enabled:=true;
  LoadFilterSettings(listboxFilters.ItemIndex);
end;

procedure TFiltersfrm.editFilterNameChange(Sender: TObject);
begin
  if listboxFilters.ItemIndex = -1 then
    exit;

  listboxFilters.Items.Strings[listboxFilters.ItemIndex]:=editFilterName.Text;
  try
    regwritestring(HKEY_CURRENT_USER,'Software\Quick And Easy\ScummVM\filters\' + keyname, 'filtername', editFilterName.Text);
  Except On EJclRegistryError do
    showmessage('Error writing ''filtername'' value!');
  end;
end;

procedure TFiltersfrm.PopulateFilters;
var
  I: Integer;
  FilterKeys: tstringlist;
begin
  filterkeys:=tstringlist.create;
  try
    reggetkeynames(HKEY_CURRENT_USER,'Software\Quick And Easy\ScummVM\Filters\',filterkeys);

    if FilterKeys.Count = 0 then
      exit;

    for I := 0 to FilterKeys.Count - 1 do
    begin
      try
        listboxfilters.Items.Add(regreadstring(HKEY_CURRENT_USER,'Software\Quick And Easy\ScummVM\Filters\' + FilterKeys.Strings[i],'filtername'));
      Except On EJclRegistryError do
        deletewonkyfilterkey(FilterKeys.Strings[i]);
      end;
    end;

  finally
    filterkeys.Free;
  end;

end;

procedure TFiltersfrm.DeleteWonkyFilterKey(keyname: string);
var
  reg: tregistry;
begin
  reg:=tregistry.Create;
  try
    reg.RootKey:=HKEY_CURRENT_USER;
    reg.DeleteKey('Software\Quick And Easy\ScummVM\Filters\' + keyname);
  finally
    reg.Free;
  end;
end;

procedure TFiltersfrm.FormShow(Sender: TObject);
begin
  listboxFilters.Clear;
  GroupBoxFilters.Enabled:=false;
  editFilterName.Enabled:=false;
  populatefilters;
end;


procedure TFiltersfrm.LoadFilterSettings(index: integer);
var
  FilterKeys: tstringlist;
begin
  filterkeys:=tstringlist.create;
  try
    reggetkeynames(HKEY_CURRENT_USER,'Software\Quick And Easy\ScummVM\Filters\',filterkeys);
    keyname:=FilterKeys.Strings[listboxFilters.itemindex];

    try
      if regreadbool(HKEY_CURRENT_USER,'Software\Quick And Easy\ScummVM\filters\' + keyname,'SCUMM')=true then
        chkboxSCUMM.Checked:=true;
    Except On EJclRegistryError do
      chkboxSCUMM.Checked:=false;
    end;

    try
      if regreadbool(HKEY_CURRENT_USER,'Software\Quick And Easy\ScummVM\filters\' + keyname,'Humongous')=true then
        chkboxHumongous.Checked:=true;
    Except On EJclRegistryError do
      chkboxHumongous.Checked:=false;
    end;

    try
      if regreadbool(HKEY_CURRENT_USER,'Software\Quick And Easy\ScummVM\filters\' + keyname,'Revolution')=true then
        chkboxRevolution.Checked:=true;
    Except On EJclRegistryError do
      chkboxRevolution.Checked:=false;
    end;

    try
      if regreadbool(HKEY_CURRENT_USER,'Software\Quick And Easy\ScummVM\filters\' + keyname,'Other')=true then
        chkboxOther.Checked:=true;
    Except On EJclRegistryError do
      chkboxOther.Checked:=false;
    end;

    try
      if regreadbool(HKEY_CURRENT_USER,'Software\Quick And Easy\ScummVM\filters\' + keyname,'Full')=true then
        chkboxFull.Checked:=true;
    Except On EJclRegistryError do
      chkboxFull.Checked:=false;
    end;

    try
      if regreadbool(HKEY_CURRENT_USER,'Software\Quick And Easy\ScummVM\filters\' + keyname,'Demos')=true then
        chkboxDemos.Checked:=true;
    Except On EJclRegistryError do
      chkboxDemos.Checked:=false;
    end;

    try
      if regreadbool(HKEY_CURRENT_USER,'Software\Quick And Easy\ScummVM\filters\' + keyname,'Installed')=true then
        chkboxInstalled.Checked:=true;
    Except On EJclRegistryError do
      chkboxInstalled.Checked:=false;
    end;

    editFilterName.Text:=listboxFilters.Items.Strings[listboxFilters.itemindex];
  finally
    filterkeys.Free;
  end;
end;

procedure TFiltersfrm.chkboxSCUMMClick(Sender: TObject);
begin
  regwritebool(HKEY_CURRENT_USER,'Software\Quick And Easy\ScummVM\filters\' + keyname,'SCUMM', chkboxSCUMM.Checked);
end;

procedure TFiltersfrm.chkboxRevolutionClick(Sender: TObject);
begin
  regwritebool(HKEY_CURRENT_USER,'Software\Quick And Easy\ScummVM\filters\' + keyname,'Revolution', chkboxRevolution.Checked);
end;

procedure TFiltersfrm.chkboxFullClick(Sender: TObject);
begin
  regwritebool(HKEY_CURRENT_USER,'Software\Quick And Easy\ScummVM\filters\' + keyname,'Full', chkboxFull.Checked);
end;

procedure TFiltersfrm.chkboxInstalledClick(Sender: TObject);
begin
  regwritebool(HKEY_CURRENT_USER,'Software\Quick And Easy\ScummVM\filters\' + keyname,'Installed', chkboxInstalled.Checked);
end;

procedure TFiltersfrm.chkboxHumongousClick(Sender: TObject);
begin
  regwritebool(HKEY_CURRENT_USER,'Software\Quick And Easy\ScummVM\filters\' + keyname,'Humongous', chkboxHumongous.Checked);
end;

procedure TFiltersfrm.chkboxOtherClick(Sender: TObject);
begin
  regwritebool(HKEY_CURRENT_USER,'Software\Quick And Easy\ScummVM\filters\' + keyname,'Other', chkboxOther.Checked);
end;

procedure TFiltersfrm.chkboxDemosClick(Sender: TObject);
begin
  regwritebool(HKEY_CURRENT_USER,'Software\Quick And Easy\ScummVM\filters\' + keyname,'Demos', chkboxDemos.Checked);
end;

end.
