//********************************
//**ScummVM Quick And Easy 1.4.0**
//**By Ben Gorman (Bgbennyboy)****
//**Http://quick.mixnmojo.com/****
//********************************

unit Scheduler;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, NTCommon, NTaskScheduler, MsTaskApi, jclregistry, jclsysinfo,
  ComCtrls, JvExComCtrls, JvDateTimePicker, CheckLst, JvExCheckLst,
  JvCheckListBox;

type
  Ttasksfrm = class(TForm)
    TaskScheduler: TNTaskScheduler;
    btnApply: TButton;
    editRunAs: TEdit;
    comboboxtype: TComboBox;
    checklistboxDays: TJvCheckListBox;
    datetimeStartOn: TJvDateTimePicker;
    datetimeStartAt: TJvDateTimePicker;
    lblStartOn: TLabel;
    lblStartAt: TLabel;
    lblType: TLabel;
    lblDays: TLabel;
    lblRunAs: TLabel;
    lblRunAsExplain: TLabel;
    editPassword: TEdit;
    lblPassword: TLabel;
    lblPasswordExplain: TLabel;
    lblHeader: TLabel;
    btnDelete: TButton;
    procedure btnApplyClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure comboboxtypeChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure checklistboxDaysClick(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
  private
    procedure LoadTaskSettings(TaskNo: integer);
    function CheckTaskExists: integer;
    { Private declarations }
  public
    { Public declarations }
  end;

var
  tasksfrm: Ttasksfrm;
  
implementation

{$R *.dfm}

function MyMessageDialog(const Msg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; Captions: array of string): Integer;
var
  aMsgDlg: TForm;
  i: Integer;
  dlgButton: TButton;
  CaptionIndex: Integer;
begin
  { Create the Dialog }
  aMsgDlg := CreateMessageDialog(Msg, DlgType, Buttons);
  captionIndex := 0;
  { Loop through Objects in Dialog }
  for i := 0 to aMsgDlg.ComponentCount - 1 do
  begin
   { If the object is of type TButton, then }
    if (aMsgDlg.Components[i] is TButton) then
    begin
      dlgButton := TButton(aMsgDlg.Components[i]);
      if CaptionIndex > High(Captions) then Break;
      { Give a new caption from our Captions array}
      dlgButton.Caption := Captions[CaptionIndex];
      Inc(CaptionIndex);
    end;
  end;
  Result := aMsgDlg.ShowModal;
  aMsgDlg.Free;// free the dialog
end;

function GetDays: word;
begin
  result:=0;
  if tasksfrm.checklistboxdays.Checked[0] then result:= TASK_SUNDAY;
  if tasksfrm.checklistboxdays.Checked[1] then result:= result or TASK_MONDAY;
  if tasksfrm.checklistboxdays.Checked[2] then result:= result or TASK_TUESDAY;
  if tasksfrm.checklistboxdays.Checked[3] then result:= result or TASK_WEDNESDAY;
  if tasksfrm.checklistboxdays.Checked[4] then result:= result or TASK_THURSDAY;
  if tasksfrm.checklistboxdays.Checked[5] then result:= result or TASK_FRIDAY;
  if tasksfrm.checklistboxdays.Checked[6] then result:= result or TASK_SATURDAY;

end;

function Ttasksfrm.CheckTaskExists: integer;
var
  i: integer;
begin
  taskscheduler.Refresh;
  result:=-1;
    for i:=0 to taskscheduler.Tasks.count -1 do
    begin
      if taskscheduler.Tasks[i].Name='Q+E Get Daily Build.job' then
      begin
        result:=i;
        break;
      end
      else
        result:=-1;
    end;
end;

procedure Ttasksfrm.LoadTaskSettings(Taskno: integer);
var
  vTask: TNTask;
  vTrigger: TNTaskTrigger;
  vMask, i: integer;
  AValue: dword;
begin
  if taskno=-1 then exit;
  vTask := TaskScheduler.Tasks[taskno];
  vTrigger:=vTask.Triggers[0];

  if vTrigger.TriggerType=TASK_TIME_TRIGGER_DAILY then
  begin
    comboboxtype.ItemIndex:=0;
    datetimeStartOn.DateTime:=vTrigger.StartTime;
    datetimeStartAt.DateTime:=vTrigger.StartTime;
    editRunAs.Text:=vTask.AccountName;
    comboboxtype.OnChange(tasksfrm);
  end
  else
  if vTrigger.TriggerType=TASK_TIME_TRIGGER_WEEKLY then
  begin
    comboboxtype.ItemIndex:=1;
    datetimeStartOn.DateTime:=vTrigger.StartTime;
    datetimeStartAt.DateTime:=vTrigger.StartTime;
    editRunAs.Text:=vTask.AccountName;

    vMask := 1;
    aValue:= vTrigger.TriggerTypeInfo.Weekly.DaysOfWeek;
    for i := 0 to 6  do
    begin
      Checklistboxdays.Checked[i] := vMask and aValue <> 0;
      vMask := vMask * 2;
    end;

    comboboxtype.OnChange(tasksfrm);
  end

end;

procedure Ttasksfrm.btnApplyClick(Sender: TObject);
var
  vTask: TNTask;
  vTrigger: TNTaskTrigger;
  temptime: tdatetime;
begin
   if CheckTaskExists >-1 then
   begin
    vTask:=TaskScheduler.Tasks[CheckTaskExists];
    vTask.Triggers.Delete(0); //delete old triggers
   end
   else
    vTask := TaskScheduler.Tasks.Add('Q+E Get Daily Build', application.exename);
             
   vTask.WorkingDirectory:=extractfilepath(application.ExeName);
   vTask.Parameters:='-U';

   if checklistboxDays.Enabled then //its weekly
   begin
    vTrigger := vTask.Triggers.Add(TASK_TIME_TRIGGER_WEEKLY);
    vTrigger.TriggerTypeInfo.Weekly.WeeksInterval:=1;
    vTrigger.TriggerTypeInfo.Weekly.DaysOfWeek := GetDays;
   end
   else //its daily
   begin
    vTrigger := vTask.Triggers.Add(TASK_TIME_TRIGGER_DAILY);
   end;

   vTrigger.Flags:=vTrigger.Flags and not TASK_TRIGGER_FLAG_DISABLED;

   temptime:=datetimeStartOn.DateTime;
   replacetime(temptime , datetimeStartAt.DateTime);
   vTrigger.StartTime:=temptime;

   vTask.SetAccountInformation(EditRunAs.Text, editpassword.Text);
   vTask.Save;
   btnDelete.Enabled:=true;
end;

procedure Ttasksfrm.FormCreate(Sender: TObject);
begin
  editRunAs.Text:=getlocalusername;
  datetimestartat.Time:=now;
  datetimestarton.Date:=now;
  
  //TaskScheduler.MachineName:='\\' + TaskScheduler.LocalComputer;
  //showmessage(TaskScheduler.LocalComputer);
end;

procedure Ttasksfrm.comboboxtypeChange(Sender: TObject);
var
  i: integer;
begin
  case  comboboxtype.ItemIndex of
    -1: exit;
     0: begin
          checklistboxdays.Enabled:=false;
          btnApply.Enabled:=true;
        end;
     1: begin
          checklistboxdays.Enabled:=true;
          for i:=0 to 6 do
          if checklistboxdays.Checked[i] then
          begin
            btnApply.Enabled:=true;
            exit;
          end
          else
            btnApply.Enabled:=false;
        end;
  end;
end;

procedure Ttasksfrm.FormShow(Sender: TObject);
begin
  taskscheduler.MachineName:='';
  try
    TaskScheduler.MachineName:='\\' + TaskScheduler.LocalComputer;
  except
    showmessage(TaskScheduler.LocalComputer);
  end;
  //showmessage(TaskScheduler.LocalComputer);

  if CheckTaskExists>-1 then
  begin
    btnDelete.Enabled:=true;
    loadtasksettings(CheckTaskExists);
  end;
end;

procedure Ttasksfrm.checklistboxDaysClick(Sender: TObject);
var
  i: integer;
begin
  for i:=0 to 6 do
  if checklistboxdays.Checked[i] then
  begin
    btnApply.Enabled:=true;
    exit;
  end
  else
    btnApply.Enabled:=false;
end;

procedure Ttasksfrm.btnDeleteClick(Sender: TObject);
begin
  if checktaskexists < 0 then exit
  else
  begin
    taskscheduler.Tasks.Delete(CheckTaskExists);
    btndelete.Enabled:=false;
  end;

end;

end.
